@@ -1,4 +1,4 @@
-# $Id: Lite.pm,v 1.10 2004/01/29 18:52:58 davidb Exp $
+# $Id: Lite.pm,v 1.11 2004/02/24 00:34:41 davidb Exp $
 #
 # Copyright (C) 2003 Verisign, Inc.
 #
@@ -36,7 +36,7 @@ our @ISA = qw(Exporter);
 our %EXPORT_TAGS = ( 'all' => [ qw(beep_listen), qw(beep_connect) ] );
 our @EXPORT_OK = ( @{ $EXPORT_TAGS{'all'} } );
 
-our $VERSION = '0.03';
+our $VERSION = '0.04';
 
 our $errorstr;
 our $debug;
@@ -287,17 +287,23 @@ sub beep_connect {
       $host = $val;
       next;
     };
+    /^Socket$/io and do {
+      $socket = $val;
+      next;
+    };
   }
 
-  $socket = new IO::Socket::INET(PeerAddr => $host,
+  if (! $socket) {
+    $socket = new IO::Socket::INET(PeerAddr => $host,
 				 PeerPort => $port,
 				 Proto    => 'tcp')
-    || do {
-      $errorstr = "could not connect to $host:$port: $!";
-      return undef;
+      || do {
+        $errorstr = "could not connect to $host:$port: $!";
+        return undef;
     };
 
-  $args{Socket} = $socket;
+    $args{Socket} = $socket;
+  }
   return Net::BEEP::Lite::ClientSession->new(%args);
 }
 
